/**************************************************************************/
/*** This file contains time-history and frequency diagnostics. Includes: */
/*** TIME HISTORY                                                         */
/*** mccrate - collision rates                                            */
/***                                                                      */
/**************************************************************************/
#include "pdc1.h"

FILE *Output;

void freqanalysis();
void realft(SCALAR [], int, int);
void velocity();
SCALAR mu_calc();

/***************************************************************************/ 
/* time history accumulator; calculates and stores all history values, and */
/* performs combing on history values when low on memory                   */
/***************************************************************************/

void history()
{
  int i, ii, j, isp; //k
  static SCALAR sntos, fscale, ef_norm;
  SCALAR power, temp, Iz_2; //temq
  //SCALAR vz_sum;

  static int count=1, navecount=0;
  static int d_skip=1;
  /* static int icount=0; */
  static SCALAR pow_avg=0.0, jnorm[NSMAX];
  //static SCALAR mu_3;

  /* Calculate normalisation constants */
  if (rescale_on) 
  {
    sntos=ECHARGE*nc2p/(2.0*PI*dr*height); /* Normalisation for sigma diag */
    fscale = 0.5*PI*height*dr*dr/ptopn/ECHARGE;
    for (isp=0; isp<nsp; isp++)
      jnorm[isp] = charge[isp]*nc2p/(PI*dr*height*dt);
    ef_norm = 1.0/(ptopn*dr);
    rescale_on = 0;
  }

/* Renormalise current */
  I_rf*=intoi[0];

/* Calculate power  */
  power = I_rf*phi[0]/ptopn;                
  if (it[0]) pow_avg = (pow_avg*(it[0]-1) + power)/((SCALAR)it[0]);

/* Normalise charge density */
  for(i=0; i<ng; i++) 
  {
	 rho[i] *= dntod*ECHARGE;
	 rho_unfilt[i] *= dntod*ECHARGE;
  }

/* Normalise density for plotting and calculate ke and j profiles*/
  for(isp=0; isp<nsp; isp++) {
    for(i=0; i<ng; i++) density[isp][i] = sp_n[isp][i]*dntod;
    if (k_count[isp] == 1) {
      for(i=0; i<ng; i++) {
         ke_r[isp][i] *= Escale[isp];
         ker_r[isp][i] *= Escale[isp];
         ket_r[isp][i] *= Escale[isp];
         kez_r[isp][i] *= Escale[isp];

         j_r[isp][i] *= jnorm[isp];
	 j_t[isp][i] *= jnorm[isp];
         j_z[isp][i] *= jnorm[isp];
/*       jr_unfilt[isp][i] *= jnorm[isp];
         jz_unfilt[isp][i] *= jnorm[isp]; */
      }

/**** Store angular momentum */
      for (i=0; i<np[isp]; i++) 
	angular_mtm[isp][i] = v_theta[isp][i]*r[isp][i];
    } /* end if (k_count ...) loop */
  }

/* Calc total axial current using both sum of v_z (1) and integration of J_z (2) */
  Iz_2 = 0.0;
  for (isp=0; isp<nsp; isp++)
      for (ii=0; ii<ng; ii++) Iz_2 += j_z[isp][ii]*r_array[ii]*dr_n[ii];
  Iz_2 *= TWOPI*dr;
  
  /*if (src !='P' && src !='p' && src !='J' && src!='j')  
  {
      vz_sum = 0;
      for (ii=0; ii<np[0]; ii++) vz_sum += v_z[0][ii];
      I_z = charge[0]*vscale*vz_sum*nc2p/height;
  }*/

/********* Calculate time-averaged profiles ******************/
  if (RT_flag && it_rate) {
    for(i=0; i<2; i++)
      for (j=0; j<nc_RT; j++)
        ex_rate_show[i][j] = (ex_rate_show[i][j]*(it_rate-1)
                           + ex_mccrate[i][j])/((SCALAR)it_rate);
    for(i=0;i<nc_RT;i++) {
      prod[i]+=ex_rate_show[0][i];
      prodm[i]+=ex_rate_show[1][i];
    }
  }
  if (it[0]) {
/* Calculate collision profiles  */
     if (ecollisional || icollisional){	
       for (i=0; i<ncolls; i++)
         for (j=0; j<ng; j++) 
           rate_show[i][j] = (rate_show[i][j]*(it[0]-1) +
                             mccrate[i][j])/((SCALAR)it[0]);
          /* c.f CVS version,
           rate_show[i][j] = (rate_show[i][j]*(it[0]-1) +
	   mccrate[i][j]*dntod/dt)/((SCALAR)it[0]); */
       for (j=0; j<ng; j++) {
         Pel_show[j] = (Pel_show[j]*(it[0]-1) + 
                       ECHARGE*Pel[j]*dntod/dt)/((SCALAR)it[0]);
         Pie_show[j] = (Pie_show[j]*(it[0]-1) +
                       ECHARGE*Pie[j]*dntod/dt)/((SCALAR)it[0]);
         Pcx_show[j] = (Pcx_show[j]*(it[0]-1) +
                       ECHARGE*Pcx[j]*dntod/dt)/((SCALAR)it[0]);
       }
  
/*       if (RT_flag) for (i=0;i<nc_RT;i++) 
       {
         k=i*nc_ratio;
         temp=temq=0;
         for (j=0;j<nc_ratio;j++) 
         {
           temp+=rate_show[1][k]+rate_show[1][k+1];   
           temq+=rate_show[6][k]+rate_show[6][k+1]; Density n(r)
           k++;
         }
         prod[i]+=0.5*temp/nc_ratio;
         prodm[i]+=0.5*temq/nc_ratio;
       }   */
     }

/* Calculate time-averaged profiles - J.E, K.E., J */
    for (isp=0; isp<nsp; isp++) {
      if (k_count[isp] == 1) {
/* Radial & Axial Current density profile */
	for (j=0; j<ng; j++) {
	  //	  jr_prof[isp][j] = (jr_prof[isp][j]*(it[isp]-1) + j_r[isp][j])/it[isp]; 
	  //	  jt_prof[isp][j] = (jt_prof[isp][j]*(it[isp]-1) + j_t[isp][j])/it[isp];
	  //	  jz_prof[isp][j] = (jz_prof[isp][j]*(it[isp]-1) + j_z[isp][j])/it[isp]; 
	  jdote[isp][j] = j_r[isp][j] * efield[j] * ef_norm;
	  jzdotez[isp][j] = j_z[isp][j] * E_z * ef_norm;
	}

/* Calculate electron & ion mobility profiles from Ez and Jz */
       for (i=0; i<ng; i++) {
	 if(density[isp][i]) {
	   if(E_z > 1E-5)		 {
	     mu_prof[isp][i]=(mu_prof[isp][i]*(it[isp]-1) + jz_prof[isp][i]/(ECHARGE*density[isp][i]*E_z*ef_norm))/it[isp];
	   }
	   else {
	     mu_prof[isp][i] = 0.0;
	   }
	 }
       }

/* Time-averaged J.E */
//       for (j=0; j<ng; j++) {
//	 jdote[isp][j] = (jdote[isp][j]*(it[isp]-1) + (j_r[isp][j]+j_r[isp][j]+j_z[isp][j])*efield[j]*ef_norm)/((SCALAR)it[isp]);
//	 jzdotez[isp][j] = (jzdotez[isp][j]*(it[isp]-1) + j_z[isp][j]*E_z*ef_norm)/((SCALAR)it[isp]);
//       }
     } /* end of if k_count = 1 */

/* Average kinetic energy profile */
//         for (j=0; j<ng; j++) {
	   //	   ke_avg[isp][j] = (ke_avg[isp][j]*(it[isp]-1) + ke_r[isp][j])/it[isp];
      //	   T_avg[isp][j] = (T_avg[isp][j]*(it[isp]-1) + Ttotal[isp][j])/it[isp];
	   //	   den2_avg[isp][j] = (den2_avg[isp][j]*(it[isp]-1) + Ttotal2[isp][j])/it[isp];
	   //	   Tr_avg[isp][j] = (Tr_avg[isp][j]*(it[isp]-1) + Tr[isp][j])/it[isp];
	   //	   Tt_avg[isp][j] = (Tt_avg[isp][j]*(it[isp]-1) + Tt[isp][j])/it[isp];
	   //	   Tz_avg[isp][j] = (Tz_avg[isp][j]*(it[isp]-1) + Tz[isp][j])/it[isp];
	   //	   ker_avg[isp][j] = (ker_avg[isp][j]*(it[isp]-1) + ker_r[isp][j])/it[isp];
	   //	   ket_avg[isp][j] = (ket_avg[isp][j]*(it[isp]-1) + ket_r[isp][j])/it[isp];
	   //	   kez_avg[isp][j] = (kez_avg[isp][j]*(it[isp]-1) + kez_r[isp][j])/it[isp];
      //	   Dr_avg[isp][j] = (Dr_avg[isp][j]*(it[isp]-1) + Dr_r[isp][j])/it[isp];
	   //	   Drr_avg[isp][j] = (Drr_avg[isp][j]*(it[isp]-1) + Drr_r[isp][j])/it[isp];
	   //	   Drt_avg[isp][j] = (Drt_avg[isp][j]*(it[isp]-1) + Drt_r[isp][j])/it[isp];
	   //	   Drz_avg[isp][j] = (Drz_avg[isp][j]*(it[isp]-1) + Drz_r[isp][j])/it[isp];
	   //         }


/* Time-averaged mid-plasma energy distribution function */
	 for (i=0; i<nbin_mid[isp]; i++)
	   fe_mid_show[isp][i] = (fe_mid_show[isp][i]*(it[0]-1) +
				  fe_mid[isp][i]/sqrt(emid_array[isp][i]+DBL_MIN))/((SCALAR)it[0]);
    }
  }

/* Determine velocity distribution */
  if (vel_dist_accum) velocity();
  
  /* Determine electron mobility using 3rd method */
  /*if (k_count[0] == 1) mu_3 = mu_calc();*/
  
  /* Store evolution of current, mid and electrode potential and power for fft */
  if(nfft)
  {
      if (thist_hi >= nfft)
      {
	  freqanalysis();
	  thist_hi=0;
      }
      cur_hist[thist_hi]= I_rf;
      phi_hist[0][thist_hi] = phi[0];
      phi_hist[1][thist_hi] = phi[nc/2];
      pow_hist[thist_hi] = power;
      Local_t_array[thist_hi] = atime;
      thist_hi++;

  /*- Time-averaged species densities -----*/

      if(navecount == nfft) {    
	  for(isp=0; isp<nsp; isp++)
	    for(i=0; i<ng; i++){
	      den_ave[isp][i]= den_sum[isp][i]/nfft; 
	      if(den_sum[isp][i]){
		ke_avg[isp][i] = ke_sum[isp][i]/den2_sum[isp][i];
		ke_sum[isp][i] = 0;
		Dr_avg[isp][i] = Dr_sum[isp][i]/nfft;//den2_sum[isp][i];//den2_sum[isp][i];//den2_sum[isp][i];//nc2p/charge[isp]);
                Drr_avg[isp][i] = 0;
		Drt_avg[isp][i] = 0;
		Drz_avg[isp][i] = 0;
		//		Dr_avg[isp][i] *= mass[isp] /2/ECHARGE;
		Dr_sum[isp][i] = 0;
		T_avg[isp][i] = (ke_avg[isp][i]-Dr_avg[isp][i])*2/3;
		T_sum[isp][i] = 0;

	      }
	      else ke_avg[isp][i] = T_avg[isp][i] = Dr_avg[isp][i] = 0;
	      den2_sum[isp][i] = 0;
	      jdote_avg[isp][i] = jdote_sum[isp][i]/nfft;
	      jdote_sum[isp][i] = 0;
	      jzdotez_avg[isp][i] = jzdotez_sum[isp][i]/nfft;
	      jzdotez_sum[isp][i] = 0;
	      jr_prof_avg[isp][i] = jr_prof_sum[isp][i]/nfft;
	      jr_prof_sum[isp][i] = 0;
	      den_sum[isp][i] = 0;
	    }

	  for(i=0; i<ng; i++) {
  /*- Average charge density profile -----*/
	    rho_ave[i] = rho_sum[i]/nfft;
	    rho_sum[i] = 0;

  /*- Time-averaged potential and electric field -----*/
	    e_ave[i] = e_sum[i]/nfft;      
	    e_sum[i] = 0;
	    phi_ave[i] = phi_sum[i]/nfft;  
	    phi_sum[i] = 0;
        }
        navecount = 0;
     }
 /* Sum up quantities used for doing averages */
      else {

/* Sum density for plotting average */
	for(isp=0; isp<nsp; isp++){
	  for(i=0; i<ng; i++){
	    den2_sum[isp][i] += sp_n_k[isp][i];
	    den_sum[isp][i]+= density[isp][i];
	    ke_sum[isp][i] += ke_r[isp][i];
	    jdote_sum[isp][i] += jdote[isp][i];
	    jzdotez_sum[isp][i] += jzdotez[isp][i];
	    jr_prof_sum[isp][i] += j_r[isp][i];
	    //	    Dr_sum[isp][i] += SQUARE(j_r[isp][i])+SQUARE(j_t[isp][i])+SQUARE(j_z[isp][i]);
	
            if (sp_n_k[isp][i]){
	    //      Dr_sum[isp][i] += (SQUARE(Drr_avg[isp][i])+SQUARE(Drt_avg[isp][i])+SQUARE(Drz_avg[isp][i]))*Escale[isp]/sp_n_k[isp][i];
	      Drr_avg[isp][i] += SQUARE(j_r[isp][i]*vol[i]/jnorm[isp]/sp_n_k[isp][i]);
	      Drt_avg[isp][i] += SQUARE(j_t[isp][i]*vol[i]/jnorm[isp]/sp_n_k[isp][i]);
	      Drz_avg[isp][i] += SQUARE(j_z[isp][i]*vol[i]/jnorm[isp]/sp_n_k[isp][i]);
	      Dr_sum[isp][i] = (Drr_avg[isp][i]+Drt_avg[isp][i]+Drz_avg[isp][i])*Escale[isp];//*sp_n_k[isp][i];
	    }
	    else Dr_sum[isp][i] = 0;
	    T_sum[isp][i] += ke_sum[isp][i] - Dr_sum[isp][i];
	    if (sp_n_k[isp][i])
	      ke_r[isp][i] /= sp_n_k[isp][i];
	    else ke_r[isp][i]=0;
	  }
	}
/* Sum charge density */
      for(i=0; i<ng; i++) rho_sum[i]+= rho[i];

 /*- Instantaneous potential and E field profiles ----*/
      for (i=0; i<ng; i++) {
        e_sum[i] += efield[i];
        phi_sum[i] += phi[i];
      }
      navecount++;
    }    /* end of "else  */
  }      /* end of "if (nfft)"*/

  /*------------Check # diagnostics stored -------------------------*/

  if (--count) return;	  /* only accum every d_skip steps */
  if (hist_hi >= HISTMAX) /* comb time histories */
  {
     for (isp=0; isp<nsp; isp++)
     {
       comb(np_hist[isp]);
       comb(np_hist2[isp]);
       comb(kes_hist[isp]);
       comb(Ts_hist[isp]);
       comb(us_hist[isp]);
       comb(usr_hist[isp]);
       comb(ust_hist[isp]);
       comb(usz_hist[isp]);
     }
     comb(LHSphi_hist);
     comb(midphi_hist);
     comb(Isheathphi_hist);
     comb(Osheathphi_hist);
     comb(IOphi_ratio_hist);
     comb(com_power_hist);
     comb(avg_pow_hist);
     comb(com_cur_hist);
     comb(sigma_hist);
     comb(ese_hist);
     comb(Isheath);
     comb(Osheath);
     comb(plasmawidth);
     comb(ke_tot);
     comb(te_hist);
     comb(Ez_hist);
     comb(Iz_hist);
     comb(Iz_hist2);
     comb(mu_hist);
     comb(mu_hist2);
     //     comb(mu_hist3);
     //     comb(nu_iz_hist);
     //     comb(nu_ex_hist);
     //     comb(nu_el_hist);
     comb(t_array);
     comb(C_hist[0]);   /*LEEHJ*/
     comb(C_hist[1]);
     hist_hi = HISTMAX/4;
     d_skip *= 4;
   #ifdef MPI_1D
     if(my_rank == ROOT)
       printf("dskip = %d \n",d_skip);
   #else
     printf("dskip = %d \n",d_skip);
   #endif
  }

  /* --------- Accumulate Histories ------------------------ */

  ke_tot[hist_hi] = 0;
  for (isp=0; isp<nsp; isp++)  
  {
/* particle flux to outer wall */
/*    flux[isp][hist_hi] = n_lost[isp]*nc2p/(dt*sp_k[isp]*area1); */

    np_hist[isp][hist_hi] = np[isp]*nc2p/reactor_vol;      /* Simulation particle numbers */
    np_hist2[isp][hist_hi] = np[isp];

/* Average kinetic energy for each species */
    if (np[isp]){
      kes_hist[isp][hist_hi] = ke_sp[isp]*Escale[isp]/np[isp];
      Ts_hist[isp][hist_hi] = T_sp[isp]*Escale[isp]/np[isp];
      us_hist[isp][hist_hi] = u_s[isp]*Escale[isp]/np[isp];
      usr_hist[isp][hist_hi] = ur_s[isp]*Escale[isp]/np[isp];
      ust_hist[isp][hist_hi] = ut_s[isp]*Escale[isp]/np[isp];
      usz_hist[isp][hist_hi] = uz_s[isp]*Escale[isp]/np[isp];
    }

/* Total kinetic energy, summed over species */
    ke_tot[hist_hi] += ke_sp[isp]*Escale[isp];

  }
  t_array[hist_hi]= atime;
  com_cur_hist[hist_hi]= I_rf;                      /* Total Current */
  LHSphi_hist[hist_hi] = phi[0];                /* LHS Potential */
  midphi_hist[hist_hi] = phi[nc/2];             /* Mid Potential */
  com_power_hist[hist_hi] = power;                /* Instantaneous Power */
  avg_pow_hist[hist_hi] = pow_avg;                /* Instantaneous Power */
  sigma_hist[hist_hi] = sigma*sntos;              /* Charge density on LH electrode */
  ese_hist[hist_hi] = r0*sigma*phi[0]*fscale;
  for (i=0; i< ng; i++) 
    ese_hist[hist_hi] += vol[i]*rho[i]*phi[i]*fscale;
  Ez_hist[hist_hi] = E_z;                           /* Axial electric field */
  mu_hist[hist_hi] = mu_e;                                /* Electron mobility */
  
  if(fabs(E_z) > 1E-5)
    {
      mu_hist2[hist_hi] = ECHARGE*nc2p*np[0]*E_z*ef_norm;
      mu_hist2[hist_hi] = height*I_z/mu_hist2[hist_hi]; /* Alt electron mobility */      
    }
  else
    {
      mu_hist2[hist_hi] = 0;
    }	      //	      den2_avg[isp][i]= den2_sum/den_ave[isp][i]/nfft;
	      //	      den2_avg[isp][i] = den2_sum[isp][i]/den_ave[isp][i];

  /*mu_hist3[hist_hi] = mu_3;*/                                /* Electron mobility */
  Iz_hist[hist_hi] = I_z;                          /* Axial current - method 1 */
  Iz_hist2[hist_hi] = Iz_2;                        /* Axial current - method 2*/
/*  nu_iz_hist[hist_hi] = nu_iz/(sp_k[0]*dt)/np[0]; */ /* Ionization rate per electron */
/*   nu_ex_hist[hist_hi] = nu_ex/(sp_k[0]*dt)/np[0]; */ /* Excitation rate per electron */
/*   nu_el_hist[hist_hi] = nu_el/(sp_k[0]*dt)/np[0]; */ /* Elastic coll rate per electron */

  te_hist[hist_hi] = ke_tot[hist_hi]+ese_hist[hist_hi]; /* Total energy */
  /* ke_tot[hist_hi] = log10(fabs(ke_tot[hist_hi]+DBL_MIN));  */                 /* kinetic energy */
  /* ese_hist[hist_hi]= log10(fabs(ese_hist[hist_hi]+DBL_MIN));  */              /* field potential */

  C_hist[0][hist_hi]=C_hist[1][hist_hi]=0;                      /* LEEHJ */
  if (RT_flag) for (i=0;i<nc_RT;i++) 
  {
    temp=2.0*i+1;
    C_hist[0][hist_hi]+=sp_ex[i]*temp;
    C_hist[1][hist_hi]+=sp_exm[i]*temp;
  }
  temp=(SCALAR)(nc_RT); temp*=temp;
  C_hist[0][hist_hi]*=dntod/temp;
  C_hist[1][hist_hi]*=dntod/temp;

  /* ------ Calculate sheath width using ne/ni = 1/2                 */
  /* ii = 0;*/

  for (ii = 0; ii < nc; ii++) {
    if (sp_n[1][ii] && sp_n[1][ii+1])
     if (sp_n[0][ii]/sp_n[1][ii] + sp_n[0][ii+1]/sp_n[1][ii+1] >= 1.2) break;
  }
  
  /* while ((sp_n[0][ii]/sp_n[1][ii] + sp_n[0][ii+1]/sp_n[1][ii+1] < 1.2) && (ii<nc/2)) ii++; */
  
  Isheath[hist_hi] = (r0+ii)*dr;          /* position of inner sheath */
  Isheathphi_hist[hist_hi] = phi[ii]-phi[0];  /* potential drop across inner sheath */
 /* ii = nc;*/
  
  for (ii = nc; ii > 0; ii--) {
    if (sp_n[1][ii] && sp_n[1][ii-1])
      if (sp_n[0][ii]/sp_n[1][ii] + sp_n[0][ii-1]/sp_n[1][ii-1] >= 1.2) break;
  }
  
  /* while ((sp_n[0][ii]/sp_n[1][ii]+ sp_n[0][ii-1]/sp_n[1][ii-1] < 1.2) && (ii>nc/2)) ii--; */

  Osheath[hist_hi] = (r0+ii)*dr;           /* position of outer sheath */
  Osheathphi_hist[hist_hi] = phi[ii]-phi[nc];     /* potential drop across outer sheath */
  IOphi_ratio_hist[hist_hi] = Isheathphi_hist[hist_hi]/Osheathphi_hist[hist_hi];   /* ratio of potential drop across left and right*/
  if (IOphi_ratio_hist[hist_hi] > 1e10) IOphi_ratio_hist[hist_hi] = 1e10;/*	  for(isp=0; isp<nsp; isp++)
	    for(i=0; i<ng; i++){
	      den_ave[isp][i]= den_sum[isp][i]/nfft; 
	      den2_avg[isp][i] = den2_sum[isp][i]/den_ave[isp][i];
	      den_sum[isp][i] = density[isp][i];
	      ke_avg[isp][i] = ke_sum[isp][i]/nfft;
	      ke_sum[isp][i] = ke_r[isp][i];
	      T_avg[isp][i] = T_sum[isp][i]/nfft;
	      T_sum[isp][i] = Ttotal[isp][i];
	      Dr_avg[isp][i] = Dr_sum[isp][i]/nfft;
	      Dr_sum[isp][i] = Dr_r[isp][i];
	      den2_sum[isp][i] = Ttotal2[isp][i];
	    }

	    for(i=0; i<ng; i++) {*/
  else if (IOphi_ratio_hist[hist_hi] < 1e-10) IOphi_ratio_hist[hist_hi] = 1e-10;
  plasmawidth[hist_hi] = Osheath[hist_hi]-Isheath[hist_hi]; /* calculate plasma width */

  hist_hi++;
  count = d_skip;
}


/****************************/

/*  When arrays are filled to [HISTMAX], takes every 4 timesteps and put into array, taking the first HISTMAX/4 indices */
void comb(SCALAR *in){
  int i, k;
  for (i=0, k=0; k<=HISTMAX; i++, k+=4){
     in[i] = in[k];
  }
}


/****************************/

/*SCALAR avg(SCALAR in)
     int navecount;
     SCALAR avg, sum;

      if(navecount == nfft) {    
	avg = sum/nfft;
	sum = in;
      }
      else {
	sum += in;
      }
return avg;
*/
  /*- Average charge density profile -----*/
//	    rho_ave[i] = rho_sum[i]/nfft;
//	    rho_sum[i] = rho[i];

  /*- Time-averaged potential and electric field -----*/
/*    e_ave[i] = e_sum[i]/nfft;      
	    e_sum[i] = efield[i];
	    phi_ave[i] = phi_sum[i]/nfft;  
	    phi_sum[i] = phi[i];
        }
        navecount = 0;
*/  /* }*/
 /* Sum up quantities used for doing averages */
//    else {

/* Sum density for plotting average */
/*	for(isp=0; isp<nsp; isp++){
	  for(i=0; i<ng; i++){
	    den2_sum[isp][i] += Ttotal2[isp][i];
	    den_sum[isp][i]+= density[isp][i];
	    ke_sum[isp][i] += ke_sum[isp][i];
	  }
	  }*/
/* Sum charge density */
//      for(i=0; i<ng; i++) rho_sum[i]+= rho[i];

 /*- Instantaneous potential and E field profiles ----*/
/*    for (i=0; i<ng; i++) {
        e_sum[i] += efield[i];
        phi_sum[i] += phi[i];
      }
      navecount++;
      }*/    /* end of "else  */


/***************************************************************/

void freqanalysis(void)
{
  int i, j, init_freq_flag=1;
  static SCALAR *temp1, *temp2;
  
  if(init_freq_flag) 
  {
    if( !(temp1= (SCALAR  *)malloc((nfft/2)*sizeof(SCALAR)))
      || !(temp2= (SCALAR  *)malloc((nfft/2)*sizeof(SCALAR))))
       #ifdef MPI_1D
         if (my_rank == ROOT)
	   puts("Null ptr returned in freqanalysis()");
       #else
         puts("Null ptr returned in freqanalysis()");
       #endif
    init_freq_flag=0;
  }
  
   /******************************************************/
  for(i=0; i< nfft; i++)
    {
      cur_fft[i]= cur_hist[i];
      phi_fft[i]= phi_hist[0][i];
      mphi_fft[i]= phi_hist[1][i];
      pow_fft[i]= pow_hist[i];
    } 
  
  realft(phi_fft-1, freq_hi, 1);
  realft(mphi_fft-1,freq_hi, 1);
  realft(cur_fft-1, freq_hi, 1);
  realft(pow_fft-1, freq_hi, 1);
  
   /******************************************************/
  /**** Computing mag and phase of the current signal ***/

  temp2[0]= (cur_fft[0] > 0.0) ? 0.0 : 180.0;
  temp1[0]= fabs(cur_fft[0])*0.5/freq_hi;
/* Calculate amplitude = sqrt(A(2j)^2 + A(2j+1)^2) & phase = atan(A(2j)/A(2j+1))   */
  for(i=1, j=2; i< freq_hi; i++, j+=2)
    {
      temp1[i]= sqrt(cur_fft[j]*cur_fft[j]+ cur_fft[j+1]*cur_fft[j+1])/freq_hi;
      if(fabs(cur_fft[j+1]) < 1e-30 && fabs(cur_fft[j]) < 1e-30)
	temp2[i]= 0.0;
      else
	temp2[i]= (180./PI)*atan2(cur_fft[j], cur_fft[j+1]);
    }

/* Store amplitude and phase in curr_fft array */
  for(i=0; i< freq_hi; i++)
    {
      cur_fft[i]= temp1[i];
      cur_fft[freq_hi+i]= temp2[i];
    }
  
   /******************************************************/
  /**** Computing mag and phase of the voltage signal ***/
  temp2[0]= (phi_fft[0] > 0.0) ? 0.0 : 180.0;


  temp1[0]= fabs(phi_fft[0])/freq_hi/2;
  for(i=1, j=2; i< freq_hi; i++, j+=2)
    {
      temp1[i]= sqrt(phi_fft[j]*phi_fft[j]+ phi_fft[j+1]*phi_fft[j+1])/freq_hi;
      if(fabs(phi_fft[j+1]) < 1e-30 && fabs(phi_fft[j]) < 1e-30)
	temp2[i]= 0.0;
      else
	temp2[i]= (180./PI)*atan2(phi_fft[j], phi_fft[j+1]);
    }
  for(i=0; i< freq_hi; i++)
    {
      phi_fft[i]= temp1[i];
      phi_fft[freq_hi+i]= temp2[i];
    }
  
   /******************************************************/
  /**** Computing mag and phase of the mid-potential signal ***/
  temp2[0]= (mphi_fft[0] > 0.0) ? 0.0 : 180.0;
  temp1[0]= fabs(mphi_fft[0])/freq_hi/2;
  for(i=1, j=2; i< freq_hi; i++, j+=2)
    {
      temp1[i]= sqrt(mphi_fft[j]*mphi_fft[j]+ mphi_fft[j+1]*mphi_fft[j+1])/freq_hi;
      if(fabs(mphi_fft[j+1]) < 1e-30 && fabs(mphi_fft[j]) < 1e-30)
	temp2[i]= 0.0;
      else
	temp2[i]= (180./PI)*atan2(mphi_fft[j], mphi_fft[j+1]);
    }
  for(i=0; i< freq_hi; i++)
    {
      mphi_fft[i]= temp1[i] + DBL_MIN;
      mphi_fft[freq_hi+i]= temp2[i];
    }
  
   /******************************************************/
  /**** Computing mag and phase of the Plasma Impedance  ***/
  for(i=0; i< freq_hi; i++)
    {
      z_fft[i]= phi_fft[i] -cur_fft[i];
      z_fft[i+freq_hi]= phi_fft[i+freq_hi] -cur_fft[i+freq_hi];
      
      if(z_fft[i+freq_hi] > 180.0)
	z_fft[i+freq_hi] -= 360.0;
      
      else if(z_fft[i+freq_hi] < -180.0)
	z_fft[i+freq_hi] += 360.0;
    }
   /******************************************************/
  /**** Computing mag and phase of Plasma Power ***/
  temp2[0]= (pow_fft[0] > 0.0) ? 0.0 : 180.0;
  temp1[0]= fabs(pow_fft[0])/freq_hi/2;
  for(i=1, j=2; i< freq_hi; i++, j+=2)
    {
      temp1[i]= sqrt(pow_fft[j]*pow_fft[j]+ pow_fft[j+1]*pow_fft[j+1])/freq_hi;
      if(fabs(pow_fft[j+1]) < 1e-30 && fabs(pow_fft[j]) < 1e-30)
	temp2[i]= 0.0;
      else
	temp2[i]= (180./PI)*atan2(pow_fft[j], pow_fft[j+1]);
    }
  for(i=0; i< freq_hi; i++)
    {
      pow_fft[i]= temp1[i] + DBL_MIN;
      pow_fft[freq_hi+i]= temp2[i];
    }
  
}

/*-------------------------------------------------------------*/
void velocity()

/* This routine stores the velocity distribution at a specified position in the discharge */
{
   int i, isp, index, dum1;
   static int initflag=1;
   static SCALAR dv_r[NSMAX], roffset[NSMAX],dv_th[NSMAX], thoffset[NSMAX],
   dv_z[NSMAX], zoffset[NSMAX];

/********** Initialise velocity arrays **********/
   if(initflag) 
   {
      initflag =0;
      for(isp=0; isp<nsp; isp++) 
      {
         dv_r[isp] = (vr_u[isp] - vr_l[isp])/((nvrbin[isp]-1)*vscale);
         roffset[isp]= vr_l[isp]/vscale/dv_r[isp];
         dv_th[isp] = (vt_u[isp] - vt_l[isp])/((nvtbin[isp]-1)*vscale);
         thoffset[isp]= vt_l[isp]/vscale/dv_th[isp];
         dv_z[isp] = (vz_u[isp] - vz_l[isp])/((nvzbin[isp]-1)*vscale);
         zoffset[isp]= vz_l[isp]/vscale/dv_z[isp];
      }
      for(isp=0; isp<nsp;isp++)
      {
           for(i=0; i<nvrbin[isp]; i++) vr_array[isp][i] = vr_l[isp]+i*dv_r[isp]*vscale;
           for(i=0; i<nvtbin[isp]; i++) vth_array[isp][i] = vt_l[isp]+i*dv_th[isp]*vscale;
           for(i=0; i<nvzbin[isp]; i++) vz_array[isp][i] = vz_l[isp]+i*dv_z[isp]*vscale;
      }
   }

/********** Store velocities **********/

   for(isp=0;isp<nsp;isp++)
   {
      if (nvrbin[isp])
      for(i=0; i<np[isp];i++)
      { 
          index = (int)(v_r[isp][i]/dv_r[isp] - roffset[isp] +0.5);
          if((index>=0) && (index < nvrbin[isp]))
          {
/*              dum1 = (int) (r[isp][i] + 0.499); */
              dum1 = (int) (j_pointer(isp,i) + 0.499);
              vr_dist[isp][dum1][index] += 1;
          }
      }
      if (nvtbin[isp])
          for(i=0; i<np[isp];i++)
          {
              index = (int)(v_theta[isp][i]/dv_th[isp] - thoffset[isp] +0.5);
              if((index>=0) && (index < nvtbin[isp]))
              {
                /* dum1 = (int) (r[isp][i] + 0.499); */
                 dum1 = (int) (j_pointer(isp,i) + 0.499);
                 vth_dist[isp][dum1][index] += 1;
              }
          }
       if (nvzbin[isp])
           for(i=0; i<np[isp];i++)
           {
               index = (int)(v_z[isp][i]/dv_z[isp] - zoffset[isp] +0.5);
               if((index>=0) && (index < nvzbin[isp]))
               {
                  /* dum1 = (int) (r[isp][i] + 0.499); */
                   dum1 = (int) (j_pointer(isp,i) + 0.499);
                   vz_dist[isp][dum1][index] += 1;
               }
           }
   } /* end of for loop */
} /* end of function */

/*-------------------------------------------------------------*/
SCALAR mu_calc()
/* Calculate electron mobility using method 3 */
{
  int ii, ibin, ebin=300;
  static int init_flag=1;
  SCALAR ke_sum, energy, f_norm, df, mu;
  SCALAR static de_f, mu_norm, *eedf;

  if (init_flag)
  {
    eedf = (SCALAR *)malloc(ebin*sizeof(SCALAR *));
    mu_norm = -sqrt(2*ECHARGE/EMASS)/(3*gas_den);
    de_f = 30.0/ebin;
    init_flag = 0;
  }

/* zero eedf */
     for (ii=0; ii<ebin-1; ii++) eedf[ii] = 0;
     ke_sum =0;

/* Determine average eedf */
     for (ii=0; ii<np[0]; ii++)
     {
         energy = SQUARE(v_r[0][ii]) + SQUARE(v_theta[0][ii]) + SQUARE(v_z[0][ii]);
         ke_sum += energy;
         ibin = energy*Escale[0]/de_f;
         if (ibin < ebin) eedf[ibin] += 1;
         f_norm += 1;
     }
/* calc de for next time */
     de_f = ke_sum*Escale[0]/(30*np[0]);

/* normalise eedf */
     for (ii=0; ii<ebin; ii++) eedf[ii] /= sqrt((ii+0.5)*de_f);

     mu = 0.0;
     for (ii=0; ii<ebin-1; ii++)
     {
       energy = (ii+1)*de_f;
       df = eedf[ii+1] - eedf[ii];
       mu += energy*df/ (*sigmaMptr)(energy);
     }
     mu *= mu_norm/f_norm/de_f;  /* normalised mobility */

     return(mu);
}
