#include "pdc1.h"
#include <string.h>
#include <xgrafix.h>

/*********************************************************/
void XGWrite(void *, int, int, FILE *, char *); 
void XGRead(void *, int, int, FILE *, char *);
void Dump(char *);
void Restore(char *);
void Quit();

void InitWindows()
{
  int ii, isp;
  char buffer[50];

  /*********************************************/
  
/*-------- Vr - r phase space --------*/
  for (isp=0; isp<nsp; isp++) 
  {
     sprintf(buffer, "Vr-R Phase Space %d", isp+1);
     XGSet2D("linlin", "R", buffer, "closed", 300, 10, dr, dr/dt,
	  False, False, r0*dr, r1*dr, vscale*(v0[isp][1]+5*vt[isp][1]), 
          vscale*(v0[isp][0]+5*vt[isp][0]));
     XGScat2D(r[isp], v_r[isp], &np[isp], isp);
  }
/*-------- Vt - r phase space --------*/
  for (isp=0; isp<nsp; isp++) 
  {
     sprintf(buffer, "Vtheta-R Phase Space %d", isp+1);
     XGSet2D("linlin", "R", buffer, "closed", 300, 10, dr, dr/dt,
	  False, False, r0*dr, r1*dr, vscale*(-v0_th[isp]-5*vt_th[isp]), 
          vscale*(v0_th[isp]+5*vt_th[isp]));
     XGScat2D(r[isp], v_theta[isp], &np[isp], isp);
  }
/*-------- Vz - r phase space --------*/
  for (isp=0; isp<nsp; isp++) 
  {
     sprintf(buffer, "Vz-R Phase Space %d", isp+1);
     XGSet2D("linlin", "R", buffer, "closed", 300, 10, dr, dr/dt,
	  False, False, r0*dr, r1*dr, vscale*(-v0_z[isp]-5*vt_z[isp]), 
          vscale*(v0_z[isp]+5*vt_z[isp]));
     XGScat2D(r[isp], v_z[isp], &np[isp], isp);
  }
/*-------- Vtr - r phase space --------*/
  for (isp=0; isp<nsp; isp++) 
  {
     sprintf(buffer, "rVt-R Phase Space %d", isp+1);
     XGSet2D("linlin", "R", buffer, "closed", 300, 10, dr, dr/dt,
	  False, True, r0*dr, r1*dr, vscale*(v0_th[isp]+5*vt_th[isp]), 
          vscale*(v0_th[isp]+5*vt_th[isp]));
     XGScat2D(r[isp], angular_mtm[isp], &np[isp], isp);
  }
/*-------- Vz - r phase space --------*/
  /*********************************************/

 XGSet2D("linlin", "Time", "Avg Density(t)", "closed", 100, 400, 1.0, 1.0, 
         True, True,1.0,1.0,1.0,1.0);

  for (isp=0; isp<nsp; isp++) XGCurve(t_array, np_hist[isp], &hist_hi, isp);
  for (isp=0;isp<2;isp++) XGCurve(t_array, C_hist[isp], &hist_hi, isp+2);

  /*********************************************/

  XGSet2D("linlin", "Time", "Computational Particle Number(t)", "closed", 100, 400, 1.0, 1.0, 
         True, True,1.0,1.0,1.0,1.0);

  for (isp=0; isp<nsp; isp++) XGCurve(t_array, np_hist2[isp], &hist_hi, isp);

  /*********************************************/
  
  XGSet2D("linlin", "R", "E field(r)", "closed", 500, 10, 1.0, 1.0/(ptopn*dr), 
           False, True, r0*dr, r1*dr,1.0,1.0);
  
  /* field from Gauss's law integration */
  XGCurve(r_array, efield, &ng, 3);

  /*********************************************/

  XGSet2D("linlin", "R", "Potential(r)", "closed", 400, 300, 1.0, 1.0/ptopn, 
           False, True, r0*dr,r1*dr,1.0,1.0);
  
  /* potential from Gauss's law calculation */
  XGCurve(r_array, phi, &ng, 4);
	
  /*********************************************/
  /* Time-dependance axial E field */
  if (src =='J' || src=='j' || src=='P'|| src=='p') 
  {
  XGSet2D("linlin", "Time", "E field(z)", "closed", 500, 10, 1.0, 1.0/(ptopn*dr),
           True, True, 1.0, 1.0,1.0,1.0);

  XGCurve(t_array, Ez_hist, &hist_hi, 3);
  }

  /*********************************************/
  /* Time-dependance of electron mobility */
  XGSet2D("linlin", "Time", "mu_e (t) ", "closed", 500, 10, 1.0, 1.0,
           True, True, 1.0, 1.0,1.0,1.0);

  XGCurve(t_array, mu_hist,  &hist_hi, 4);
  XGCurve(t_array, mu_hist2, &hist_hi, 3);
  /*XGCurve(t_array, mu_hist3, &hist_hi, 5);*/
  /*********************************************/
  XGSet2D("linlin", "Time", "I (z)", "closed", 500, 10, 1.0, 1.0,
           True, True, 1.0, 1.0,1.0,1.0);

     XGCurve(t_array, Iz_hist, &hist_hi, 4);
     XGCurve(t_array, Iz_hist2, &hist_hi, 6);
  /*********************************************/
/*
  XGSet2D("linlin", "Time", "Ionization rate", "closed", 500, 400,
           1.0, 1.0, True, True,1.0,1.0,1.0,1.0);

  XGCurve(t_array, nu_iz_hist, &hist_hi, 4);
*/
  /*********************************************/
/*
  XGSet2D("linlin", "Time", "Excitation rate", "closed", 500, 400,
           1.0, 1.0, True, True,1.0,1.0,1.0,1.0);

  XGCurve(t_array, nu_ex_hist, &hist_hi, 4);
*/
  /*********************************************/
/*
  XGSet2D("linlin", "Time", "Elastic scatt rate", "closed", 500, 400,
           1.0, 1.0, True, True,1.0,1.0,1.0,1.0);

  XGCurve(t_array, nu_el_hist, &hist_hi, 4);
*/
  /*********************************************/

  XGSet2D("linlin", "Time", "Mid Potential(t)", "closed", 500, 400, 
           1.0, 1.0/ptopn, True, True,1.0,1.0,1.0,1.0);
  
  XGCurve(t_array, midphi_hist, &hist_hi, 4);

  /*********************************************/

  XGSet2D("linlin", "Time", "Inner Electrode Potential(t)", "closed", 700, 400, 
           1.0, 1.0/ptopn, True, True,1.0,1.0,1.0,1.0);
  
  XGCurve(t_array, LHSphi_hist, &hist_hi, 4);

  /*********************************************/
  
  XGSet2D("linlin", "Time", "Inner Sheath Potential Drop(t)", "closed", 700, 400,
	  1.0,1.0/ptopn, True, True, 1.0, 1.0, 1.0, 1.0);
  XGCurve(t_array, Isheathphi_hist, &hist_hi, 4);
  
  /*********************************************/
  
  XGSet2D("linlin", "Time", "Outer Sheath Potential Drop(t)", "closed", 700, 400,
	  1.0,1.0/ptopn, True, True, 1.0, 1.0, 1.0, 1.0);
  XGCurve(t_array, Osheathphi_hist, &hist_hi, 4);
  
  /*********************************************/
  
  XGSet2D("linlin", "Time", "Sheath Potential Drop Ratio(t)", "closed", 700, 400,
	  1.0, 1.0, True, True, 1.0, 1.0, 1.0, 1.0);
  XGCurve(t_array, IOphi_ratio_hist, &hist_hi, 5);
  
  /*********************************************/

if (src =='I' || src=='i' || src=='V'|| src=='v' || src=='K' || src=='W') {
  XGSet2D("linlin", "Time", "Current(t)", "closed", 600, 400, 1.0, 1.0, 
           True, True,1.0,1.0,1.0,1.0);
  
  XGCurve(t_array, com_cur_hist, &hist_hi, 4);

  /*********************************************/

  XGSet2D("linlin", "Time", "Sigma(t)", "closed", 500, 500, 1.0, 1.0, 
           True, True,1.0,1.0,1.0,1.0);
  
  XGCurve(t_array, sigma_hist, &hist_hi, 3);
  
  /*********************************************/

  XGSet2D("linlin", "Time", "Power(t)", "close", 300, 550, 1.0, 
    	    1.0/ptopn, True, True, 0.0, 0.0, 0.0, 0.0);

  XGCurve(t_array, com_power_hist, &hist_hi, 1);
  XGCurve(t_array, avg_pow_hist, &hist_hi, 2);
}
  /*********************************************/

 /* XGSet2D("linlin", "Time", "Flux to wall", "closed", 600, 200, 
           1.0, 1.0, True, True, 0.0, 0.0, 0.0, 0.0);
 
  for (isp=0; isp<nsp; isp++) XGCurve(t_array, flux[isp], &hist_hi, isp);*/

  /*********************************************/
  /* Charge density profile */

  XGSet2D("linlin", "R", "rho(r) filt,unfilt", "closed", 400, 100, 1.0, 1.0, 
          False, True,r0*dr,r1*dr,1.0,1.0);

  XGCurve(r_array, rho, &ng, 2);
  XGCurve(r_array, rho_unfilt, &ng, 3);

  /*********************************************/
  /* Particle density profile */
 
  XGSet2D("linlin", "R", "Density n(r)", "closed", 600, 500, 1.0, 1.0,
	  False, True, r0*dr,r1*dr,1.0,1.0);

  for (isp=0; isp<nsp; isp++) XGCurve(r_array, density[isp], &ng, isp);

  /*********************************************/
  /* J.E energy profile */
 
  XGSet2D("linlin", "R", "J-E (r)", "closed", 600, 500, 1.0, 1.0,
	  False, True, r0*dr,r1*dr,1.0,1.0);

  for (isp=0; isp<nsp; isp++) XGCurve(r_array, jdote[isp], &ng, isp);
 
  /* Avg. J.E energy profile */
 
  XGSet2D("linlin", "R", "Time Avg J-E (r)", "closed", 600, 500, 1.0, 1.0,
	  False, True, r0*dr,r1*dr,1.0,1.0);

  for (isp=0; isp<nsp; isp++) XGCurve(r_array, jdote_avg[isp], &ng, isp);

  /* Jz.Ez energy profile */
 

  /*********************************************/
  /* Jz.Ez energy profile */
  XGSet2D("linlin", "R", "Jz-Ez (r)", "closed", 600, 500, 1.0, 1.0,
	  False, True, r0*dr,r1*dr,1.0,1.0);

  for (isp=0; isp<nsp; isp++) XGCurve(r_array, jzdotez[isp], &ng, isp);
 
  /* Avg. J.E energy profile */
 
  XGSet2D("linlin", "R", "Time Avg Jz-Ez (r)", "closed", 600, 500, 1.0, 1.0,
	  False, True, r0*dr,r1*dr,1.0,1.0);

  for (isp=0; isp<nsp; isp++) XGCurve(r_array, jzdotez_avg[isp], &ng, isp);

  /*********************************************/
  /* Time-averaged Current Density profile */

  XGSet2D("linlin", "R", "Time Avg Ion J (r)", "closed", 600, 500, 1.0, 1.0,
          False, True, r0*dr,r1*dr,1.0,1.0);

  for (isp=0; isp<nsp; isp++) 

    XGCurve(r_array, jr_prof_avg[isp], &ng, isp);
  

  XGSet2D("linlin", "R", "Ion J (r)", "closed", 600, 500, 1.0, 1.0,
          False, True, r0*dr,r1*dr,1.0,1.0);

  for (isp=0; isp<nsp; isp++) 
    XGCurve(r_array, j_r[isp], &ng, isp);
  
  /*********************************************/
  /* Electron & ion mobility profile */

  XGSet2D("linlin", "R", "mu_e (r) ", "closed", 400, 100, 1.0, 1.0,
          False, True,r0*dr,r1*dr,1.0,1.0);

   for (isp=0; isp<nsp; isp++) XGCurve(r_array, mu_prof[isp], &ng, isp);

  /*********************************************/
  /* Axial current density profile */
  XGSet2D("linlin", "R", "J_z (r)", "closed", 600, 500, 1.0, 1.0,
          False, True, r0*dr,r1*dr,1.0,1.0);

  for (isp=0; isp<nsp; isp++) XGCurve(r_array, jz_prof[isp], &ng, isp);
  /* for (isp=0; isp<nsp; isp++) XGCurve(r_array, jz_unfilt[isp], &ng, isp+2); */




  /*****************************/
  /* Time-averaged KE profiles */
  /*****************************/
  XGSet2D("linlin", "R", "Time Avg KE(r)", "closed", 600, 500, 1.0, 1.0,
          False, True, r0*dr,r1*dr,1.0,1.0);
 
  for (isp=0; isp<nsp; isp++) XGCurve(r_array, ke_avg[isp], &ng, isp);
  XGSet2D("linlin", "R", "Instantaneous KE(r)", "closed", 600, 500, 1.0, 1.0,
          False, True, r0*dr,r1*dr,1.0,1.0);
 
  for (isp=0; isp<nsp; isp++) XGCurve(r_array, ke_r[isp], &ng, isp);
  /*----------------------------------*/
  XGSet2D("linlin", "R", "Time Avg KE_r(r)", "closed", 600, 500, 1.0, 1.0,
          False, True, r0*dr,r1*dr,1.0,1.0);

  for (isp=0; isp<nsp; isp++) XGCurve(r_array, ker_avg[isp], &ng, isp);
  /*----------------------------------*/
  XGSet2D("linlin", "R", "Time Avg KE_theta(r)", "closed", 600, 500, 1.0, 1.0,
          False, True, r0*dr,r1*dr,1.0,1.0);

  for (isp=0; isp<nsp; isp++) XGCurve(r_array, ket_avg[isp], &ng, isp);
  /*----------------------------------*/
  XGSet2D("linlin", "R", "Time Avg KE_z(r)", "closed", 600, 500, 1.0, 1.0,
          False, True, r0*dr,r1*dr,1.0,1.0);

  for (isp=0; isp<nsp; isp++) XGCurve(r_array, kez_avg[isp], &ng, isp);





  /*************************************/
  /* Instantenous Temperature Profiles */
  /*************************************/
  XGSet2D("linlin", "R", "Instantaneous Temperature(r)", "closed", 600, 500, 1.0, 1.0,
	  False, True, r0*dr, r1*dr, 1.0, 1.0);

  for (isp=0; isp<nsp; isp++) XGCurve(r_array, Ttotal[isp], &ng, isp);
  /*----------------------------------*/
  XGSet2D("linlin", "R", "Instantaneous Temperature_r(r)", "closed", 600, 500, 1.0, 1.0,
	  False, True, r0*dr, r1*dr, 1.0, 1.0);

  for (isp=0; isp<nsp; isp++) XGCurve(r_array, Tr[isp], &ng, isp);
  /*----------------------------------*/
  XGSet2D("linlin", "R", "Instantaneous Temperature_t(r)", "closed", 600, 500, 1.0, 1.0,
	  False, True, r0*dr, r1*dr, 1.0, 1.0);

  for (isp=0; isp<nsp; isp++) XGCurve(r_array, Tt[isp], &ng, isp);
  /*----------------------------------*/
  XGSet2D("linlin", "R", "Instantaneous Temperature_z(r)", "closed", 600, 500, 1.0, 1.0,
	  False, True, r0*dr, r1*dr, 1.0, 1.0);

  for (isp=0; isp<nsp; isp++) XGCurve(r_array, Tz[isp], &ng, isp);

  /***************************************/
  /* Instantaneous Drift Energy Profiles */
  /***************************************/

  XGSet2D("linlin", "R", "Instantaneous Drift Energy(r)", "closed", 600, 500, 1.0, 1.0,
	  False, True, r0*dr, r1*dr, 1.0, 1.0);

  for (isp=0; isp<nsp; isp++) XGCurve(r_array, Dr_r[isp], &ng, isp);

  /*----------------------------------*/

  XGSet2D("linlin", "R", "Instantaneous Drift Energy_r(r)", "closed", 600, 500, 1.0, 1.0,
	  False, True, r0*dr, r1*dr, 1.0, 1.0);

  for (isp=0; isp<nsp; isp++) XGCurve(r_array, Drr_r[isp], &ng, isp);

  /*----------------------------------*/

  XGSet2D("linlin", "R", "Instantaneous Drift Energy_t(r)", "closed", 600, 500, 1.0, 1.0,
	  False, True, r0*dr, r1*dr, 1.0, 1.0);

  for (isp=0; isp<nsp; isp++) XGCurve(r_array, Drt_r[isp], &ng, isp);
  /*----------------------------------*/

  XGSet2D("linlin", "R", "Instantaneous Drift Energy_z(r)", "closed", 600, 500, 1.0, 1.0,
	  False, True, r0*dr, r1*dr, 1.0, 1.0);

  for (isp=0; isp<nsp; isp++) XGCurve(r_array, Drz_r[isp], &ng, isp);




  /***************************************/
  /*  Time Averaged Temperature Profiles */
  /***************************************/
  XGSet2D("linlin", "R", "Time Avg Temperature(r)", "closed", 600, 500, 1.0, 1.0,
	  False, True, r0*dr, r1*dr, 1.0, 1.0);

  for (isp=0; isp<nsp; isp++) XGCurve(r_array, T_avg[isp], &ng, isp);
  /***************************************/
  /*  XGSet2D("linlin", "R", "Time Avg Temperature2(r)", "closed", 600, 500, 1.0, 1.0,
	  False, True, r0*dr, r1*dr, 1.0, 1.0);

	  for (isp=0; isp<nsp; isp++) XGCurve(r_array, den2_avg[isp], &ng, isp);*/
  /*----------------------------------*/
  XGSet2D("linlin", "R", "Time Avg Temperature_r (r)", "closed", 600, 500, 1.0, 1.0,
	  False, True, r0*dr, r1*dr, 1.0, 1.0);

  for (isp=0; isp<nsp; isp++) XGCurve(r_array, Tr_avg[isp], &ng, isp);
  /*----------------------------------*/
  XGSet2D("linlin", "R", "Time Avg Temperature_t (r)", "closed", 600, 500, 1.0, 1.0,
	  False, True, r0*dr, r1*dr, 1.0, 1.0);

  for (isp=0; isp<nsp; isp++) XGCurve(r_array, Tt_avg[isp], &ng, isp);
  /*----------------------------------*/
  XGSet2D("linlin", "R", "Time Avg Temperature_z (r)", "closed", 600, 500, 1.0, 1.0,
	  False, True, r0*dr, r1*dr, 1.0, 1.0);

  for (isp=0; isp<nsp; isp++) XGCurve(r_array, Tz_avg[isp], &ng, isp);

  /**************************************/
  /* Time-averaged Drift Energy Profile */
  /**************************************/
  XGSet2D("linlin", "R", "Time Avg Drift Energy(r)", "closed", 600, 500, 1.0, 1.0,
	  False, True, r0*dr, r1*dr, 1.0, 1.0);

  for (isp=0; isp<nsp; isp++) XGCurve(r_array, Dr_avg[isp], &ng, isp);
 /*----------------------------------*/
 
  XGSet2D("linlin", "R", "Time Avg Drift Energy_r(r)", "closed", 600, 500, 1.0, 1.0,
	  False, True, r0*dr, r1*dr, 1.0, 1.0);

  for (isp=0; isp<nsp; isp++) XGCurve(r_array, Drr_avg[isp], &ng, isp);
  /*----------------------------------*/
  XGSet2D("linlin", "R", "Time Avg Drift Energy_t(r)", "closed", 600, 500, 1.0, 1.0,
	  False, True, r0*dr, r1*dr, 1.0, 1.0);

  for (isp=0; isp<nsp; isp++) XGCurve(r_array, Drt_avg[isp], &ng, isp);
  /*----------------------------------*/
  XGSet2D("linlin", "R", "Time Avg Drift Energy_z(r)", "closed", 600, 500, 1.0, 1.0,
	  False, True, r0*dr, r1*dr, 1.0, 1.0);

  for (isp=0; isp<nsp; isp++) XGCurve(r_array, Drz_avg[isp], &ng, isp);


  /*********************************************/
  /*     Excited State Diagnostics             */
  /*********************************************/
if (RT_flag) {
  XGSet2D("linlin", "R", "Density_r (r)", "closed", 600, 500, Dr,
            dntod, False, True, 0.0, r1*dr, 0.0, 0.0);
  XGCurve(r_grid2, sp_ex, &nc_RT, 3);

  XGSet2D("linlin", "R", "Density_m (r)", "closed", 600, 500, Dr,
            dntod, False, True, 0.0, r1*dr, 0.0, 0.0);
  XGCurve(r_grid2, sp_exm, &nc_RT, 2);

  XGSet2D("linlin", "R", "Emission coeff (r)", "closed", 600, 500, Dr,
           F_factor, False, True, 0.0, r1*dr, 0.0, 0.0);
  XGCurve(r_grid2, emission_coeff, &nc_RT, 3);

  XGSet2D("linlin", "R", "Radiation Flux (r)", "closed", 600, 500, Dr,
           2*M_PI*Dr*F_factor, False, True, 0.0, r1*dr, 0.0, 0.0);
  XGCurve(r_grid2, radiation_flux, &nc_RT, 3);

  XGSet3D("linlinlin","i","j","A_ij", 45, -45, "closed", 600, 500, 1, 1, 1,
          False, False, True, 0, nc_RT, 0, nc_RT, 0, 0);
  XGSurf(r_grid, r_grid, A, &nc_RT, &nc_RT, 3);

/*XGSet3D("linlinlin", "Time", "R", "time ave N_ex(r,t)", 45.0, 225.0,
          "closed", 100, 150, 1.0, Dr, dntod, True, False, True,
          0.0, 0.0, 0.0, r1*dr, 0.0, 0.0);
  XGSurf(t_array2, r_grid2, st_nex, &hist_hi2, &nc_RT, 3);       */

  XGSet2D("linlin", "R", "Effective production (r)", "closed", 600, 500, Dr,
           dntod/(dt*sp_k_ex), False, True, 0.0, r1*dr, 0.0, 0.0);
  XGCurve(r_grid2, E_prod[0], &nc_RT, 3);

  XGSet2D("linlin", "R", "Effective production2 (r)", "closed", 600, 500, Dr,
           dntod/(dt*sp_k_ex), False, True, 0.0, r1*dr, 0.0, 0.0);
  for (isp=1;isp<4;isp++) XGCurve(r_grid2, E_prod[isp], &nc_RT, isp);

  XGSet2D("linlog", "Time", "Error_sum (t)", "closed", 100, 400, 1.0,
           1.0/nc_RT, True, False, 0.0, 0.0, 1e-7, 0.1);
  XGCurve(t_array3, error_sum, &hist_hi3, 2);
 
  XGSet2D("linlin", "R", "Error (r)", "closed", 700, 500, Dr,
            1, False, True, 0.0, r1*dr, 0.0, 0.0);
    XGCurve(r_grid2, error, &nc_RT, 3);

  XGSet2D("linlin", "Time", "Radiation Flux (t)", "closed", 600, 500, 1.0,
           2*M_PI*Dr*F_factor, True, True, 0.0, 0, 0.0, 0.0);
  XGCurve(t_array3, tot_intensity, &hist_hi3, 0);   

  XGSet2D("linlin", "normalized f", "Lineshape (f)", "closed", 600, 500, 1,
            1, False, True, xmin, xmax, 0.0, 0.0);
    XGCurve(x_array, L_array, &xbin, 3);

}
  /*********************************************/
  /* Distribution functions                    */
  /*********************************************/

  for(isp=0; isp<nsp; isp++)
  {
    sprintf(buffer, "f(E) %d", isp+1);

    XGSet2D("linlin", "E", buffer, "closed", 650, 350, 
         Escale[isp], 1.0, False, True, e_array[isp][0]*Escale[isp],
	 e_array[isp][nbin[isp]-1]*Escale[isp], 0.0, 0.0);
    XGCurve(e_array[isp], fe[isp], &nbin[isp], isp);

  }
  for(isp=0; isp<nsp; isp++)
  {
    sprintf(buffer, "f(E) mid %d", isp+1);

    XGSet2D("linlin", "E", buffer, "closed", 650, 350, 
         Escale[isp], 1.0, False, True, emid_array[isp][0]*Escale[isp],
         emid_array[isp][nbin_mid[isp]-1]*Escale[isp], 0.0, 0.0);
    XGCurve(emid_array[isp], fe_mid_show[isp], &nbin_mid[isp], isp);
  }

/*********************************************/
/* velocity distribution */
   if (vel_dist_accum)
   {
     for(isp=0; isp<nsp; isp++)
     {
      if (nvrbin[isp])
      {
          sprintf(buffer, "V_r dist species %d", isp+1);
          XGSet3D("linlinlin", "R", "Vr", buffer, -100.0, 40.0, "closed",
                700, 175, dr, 1, 1.0, True, True, True, 0, 1.0, 0, 1.0, 0, 1.0);
          XGSurf(r_grid, vr_array[isp], vr_dist[isp], &ng, &nvrbin[isp], 4);
      }
      if (nvtbin[isp])
      {
           sprintf(buffer, "V_th dist species %d", isp+1);
           XGSet3D("linlinlin", "R", "Vt", buffer, -100.0, 40.0, "closed",
                700, 175, dr, 1, 1.0, True, True, True, 0, 1.0, 0, 1.0, 0, 1.0);
           XGSurf(r_grid, vth_array[isp], vth_dist[isp], &ng, &nvtbin[isp], 4);
      }
      if (nvzbin[isp])
      {  
          sprintf(buffer, "V_z dist species %d", isp+1);
          XGSet3D("linlinlin", "R", "Vz", buffer, -100.0, 40.0, "closed",
                700, 175, dr, 1, 1.0, True, True, True, 0, 1.0, 0, 1.0, 0, 1.0);
          XGSurf(r_grid, vz_array[isp], vz_dist[isp], &ng, &nvzbin[isp], 4);
      }
    }
  } /* end if vel_dist_accum */

  /*********************************************/

  XGSet2D("linlin", "Theta", "f(theta)", "closed", 600, 350, 180/PI, 1.0,
          False, True, 0.0, 90.0, 0.0, 0.0);

  for (isp=0; isp<nsp; isp++) XGCurve(th_array[isp], ftheta[isp],
         &nbin[isp], isp);

  /*********************************************/

  XGSet2D("linlog", "Time", "Avg Kinetic Energy(t) (eV)", "closed", 500, 200,
          1.0, 1.0, True, True, 0.0, 0.0, 0.0, 0.0);

  for (isp=0; isp<nsp; isp++) XGCurve(t_array, kes_hist[isp], &hist_hi, isp);

  /*********************************************/

  XGSet2D("linlog", "Time", "Avg Temperature(t) (eV)", "closed", 500, 200,
	  1.0, 1.0, True, True, 0.0, 0.0, 0.0, 0.0);
  for (isp=0; isp<nsp; isp++) XGCurve(t_array, Ts_hist[isp], &hist_hi, isp);

  /*********************************************/

  XGSet2D("linlog", "Time", "Avg Drift Energy(t) (eV)", "closed", 500, 200,
	  1.0, 1.0, True, True, 0.0, 0.0, 0.0, 0.0);
  for (isp=0; isp<nsp; isp++) XGCurve(t_array, us_hist[isp], &hist_hi, isp);

  XGSet2D("linlog", "Time", "Avg Drift Energy_r(t) (eV)", "closed", 500, 200,
	  1.0, 1.0, True, True, 0.0, 0.0, 0.0, 0.0);
  for (isp=0; isp<nsp; isp++) XGCurve(t_array, usr_hist[isp], &hist_hi, isp);
  XGSet2D("linlog", "Time", "Avg Drift Energy_t(t) (eV)", "closed", 500, 200,
	  1.0, 1.0, True, True, 0.0, 0.0, 0.0, 0.0);
  for (isp=0; isp<nsp; isp++) XGCurve(t_array, ust_hist[isp], &hist_hi, isp);
  XGSet2D("linlog", "Time", "Avg Drift Energy_z(t) (eV)", "closed", 500, 200,
	  1.0, 1.0, True, True, 0.0, 0.0, 0.0, 0.0);
  for (isp=0; isp<nsp; isp++) XGCurve(t_array, usz_hist[isp], &hist_hi, isp);

  /*********************************************/

  XGSet2D("linlin", "Time", "Field Energy(t) (eV)", "closed", 650, 400, 1.0,
         1.0, True, True, 0.0, 0.0, 0.0, 0.0);

  XGCurve(t_array, ese_hist, &hist_hi, 4);
  XGCurve(t_array, ke_tot, &hist_hi, 5);
  XGCurve(t_array, te_hist, &hist_hi, 6);

  /*********************************************/

  XGSet2D("linlin", "Time", "Sheath(t)", "closed", 650, 400, 1.0, 1.0,
                True, True, 0.0, 0.0, r0, r0+(r1-r0)*dr);
  XGCurve(t_array, Osheath, &hist_hi, 1);
  XGCurve(t_array, Isheath, &hist_hi, 2);


  /*********************************************/

  XGSet2D("linlin", "Time", "Plasma Width(t)", "closed", 650, 400, 1.0, 1.0,
	  True, False, 0.0, 0.0, 0.0, (r1-r0)*dr);
  XGCurve(t_array, plasmawidth, &hist_hi, 6);

  /*********************************************/





/* Collision profiles */
  if(ecollisional || icollisional)
  {
     for(ii=0; ii<ncolls; ii++)
     {
        XGSet2D("linlin", "R", rate_title[ii], "closed", 100, 500, 
          1.0, dntod/dt, False, True, r0*dr, r1*dr, 0.0, 1.0);

        XGCurve(r_array, rate_show[ii], &ng, 3);
     }
/*
     if (RT_flag) {
        XGSet2D("linlin", "R", "MCC Meta", "closed", 100, 500,
                Dr dntod/dt, False, True, r0*dr, r1*dr, 0.0, 1.0);
        XGCurve(r_grid2, ex_rate_show[1], &nc_RT, 3);

        XGSet2D("linlin", "R", "MCC Rad", "closed", 100, 500,
                Dr, dntod/dt, False, True, r0*dr, r1*dr, 0.0, 1.0);
        XGCurve(r_grid2, ex_rate_show[0], &nc_RT, 3);
     }
*/
     XGSet2D("linlin", "R","Elastic Heat Loss", "closed",
              100, 500, 1.0, 1.0, False, True, r0*dr, r1*dr, 0.0, 1.0);

     XGCurve(r_array, Pel_show, &ng, 3);

     XGSet2D("linlin", "R","Inelastic Heat Loss", "closed",
              100, 500, 1.0, 1.0, False, True, r0*dr, r1*dr, 0.0, 1.0);

     XGCurve(r_array, Pie_show, &ng, 3);

     XGSet2D("linlin", "R","Charge Exchange Heat Loss", "closed",
              100, 400, 1.0, 1.0, False, True, r0*dr, r1*dr, 0.0, 1.0);

     XGCurve(r_array, Pcx_show, &ng, 3);
  }
  /*********************************************/

  if(nfft)
  {
    XGSet2D("linlin", "X", "Time Avg E field(r)", "close", 10, 500, 1.0, 
        1.0/(ptopn*dr), False, True, r0*dr,r1*dr, 0.0, 0.0);
   
    XGCurve(r_array, e_ave, &ng, 3);

    /*********************************************/

    XGSet2D("linlin", "X", "Time Avg Potential(r)", "close", 400, 300, 1.0, 
          1.0/ptopn, False, True, r0*dr,r1*dr, 0.0, 0.0);
   
    XGCurve(r_array, phi_ave, &ng, 4);

    /*********************************************/

    XGSet2D("linlin", "X", "Time Avg Density n(r)", "close", 600, 500, 1.0, 
	     1.0, False, True, r0*dr,r1*dr, 0.0, 0.0);
   
    for (isp=0; isp<nsp; isp++) XGCurve(r_array, den_ave[isp], &ng, isp);

    /*********************************************/

    /*    XGSet2D("linlin", "X", "Time Avg Temp2 (r)", "close", 600, 500, 1.0, 
	     1.0, False, True, r0*dr,r1*dr, 0.0, 0.0);
   
	     for (isp=0; isp<nsp; isp++) XGCurve(r_array, T2_avg[isp], &ng, isp); */

    /*********************************************/

    XGSet2D("linlin", "X", "Time Avg Charge n(r)", "close", 400, 300, 1.0, 1.0,
            False, True, r0*dr,r1*dr, 0.0, 0.0);
  
    XGCurve(r_array, rho_ave, &ng, 4);


    /*********************************************/

    XGSet2D("linlin", "Time", "Local Mid Potential(t)", "close", 300, 550,
            1.0, 1.0/ptopn, True, True, 0.0, 0.0, 0.0, 0.0);
   
    XGCurve(Local_t_array, phi_hist[1], &thist_hi, 3);

    /*********************************************/

    XGSet2D("linlin", "Time", "Local Inner Electrode Potential(t)", "close", 300, 550,
            1.0, 1.0/ptopn, True, True, 0.0, 0.0, 0.0, 0.0);

    XGCurve(Local_t_array, phi_hist[0], &thist_hi, 3);

    /*********************************************/

    XGSet2D("linlin", "Time", "Local Current(t)", "close", 300, 550, 1.0, 
    	    1.0, True, True, 0.0, 0.0, 0.0, 0.0);

    XGCurve(Local_t_array, cur_hist, &thist_hi, 3);

    /*********************************************/

    XGSet2D("linlog", "Freq", "Mag of Mid V(f)", "close", 100, 250, 1.0, 1.0, 
           True, True, 0.0, 0.0, 0.0, 0.0);

    XGCurve(f_array, mphi_fft, &freq_hi, 4);

    /*********************************************/

    XGSet2D("linlin", "Freq", "Phase of Mid V(f)", "close", 400, 250, 1.0,
             1.0, True, False, 1.0, 1.0, -180.0, 180.0);

    XGCurve(f_array, mphi_fft+freq_hi, &freq_hi, 4);

    /*********************************************/

    XGSet2D("linlog", "Freq", "Mag of LHS V(f)", "close", 100, 250, 1.0, 1.0, 
           True, True, 0.0, 0.0, 0.0, 0.0);

    XGCurve(f_array, phi_fft, &freq_hi, 4);

    /*********************************************/

    XGSet2D("linlin", "Freq", "Phase of LHS V(f)", "close", 400, 250, 1.0, 
            1.0, True, False, 1.0, 1.0, -180.0, 180.0);

    XGCurve(f_array, phi_fft+freq_hi, &freq_hi, 4);

    /*********************************************/

    XGSet2D("linlog", "Freq", "Mag of I(f)", "close", 100, 250, 1.0, 1.0,
        True, True, 0.0, 0.0, 0.0, 0.0);

    XGCurve(f_array, cur_fft, &freq_hi, 4);

    /*********************************************/

    XGSet2D("linlin", "Freq", "Phase of I(f)", "close", 400, 250, 1.0, 1.0,
            True, False, 1.0, 1.0, -180.0, 180.0);

    XGCurve(f_array, cur_fft+freq_hi, &freq_hi, 4);

    /*********************************************/

    XGSet2D("linlog", "Freq", "Mag of P(f)", "close", 100, 250, 1.0, 1.0,
        True, True, 0.0, 0.0, 0.0, 0.0);

    XGCurve(f_array, pow_fft, &freq_hi, 4);

    /*********************************************/

    XGSet2D("linlin", "Freq", "Phase of P(f)", "close", 400, 250, 1.0, 1.0,
            True, False, 1.0, 1.0, -180.0, 180.0);

    XGCurve(f_array, pow_fft+freq_hi, &freq_hi, 4);

    /*********************************************/
     XGSet2D("linlin", "Freq", "Phase btwn V & I", "close", 400, 250, 1.0, 1.0,
                    True, False, 1.0, 1.0, -180.0, 180.0);

      XGCurve(f_array, z_fft+freq_hi, &freq_hi, 4);

    /*********************************************/
  }
}
char Revision[]={'2','.','1'};
char RevisionR[]={'2','.','R'};

/***********************************************
** Dumps data to a file so that simulation can
** be restarted from the time of the dump
************************************************/
void Dump(char *filename)
{
  int i, j;
  SCALAR ftemp, length; // temp;
  char dropfile[50];
  FILE *DMPFile, *DropFile;
 
#ifdef MPI_1D
  int isp, source, tag, sum; 
  MPI_Status status;
  
  if (my_rank == ROOT) { /* Only ROOT proc. dumps to the file */
    if ((DMPFile = fopen(filename, "w")) == NULL) {
      puts("Dump: open failed");
      return;
    }
    
    printf("Start Dump\n");
  }

  for (isp=0; isp < nsp; isp++){/* Gather data to dump into dumpfile */
      tag = 50 + isp;
      if(my_rank == ROOT)
      {
         counts[0] = np[isp];
         displs[0] = 0;
         sum = counts[0];
         for (source = 1; source <num_procs; source++){
           MPI_Recv(&counts[source], 1, MPI_INT, source,tag,
              MPI_COMM_WORLD, &status);
           displs[source] = displs[source - 1] + counts[source-1];
           sum = sum + counts[source];
         }
         np_tot[isp] = sum;
      }
      else{
        MPI_Send(&np[isp], 1, MPI_INT, 0, tag, MPI_COMM_WORLD);
      }
      
      MPI_Gatherv(r[isp], np[isp], MPI_FLOAT, r_tot[isp], counts, 
         displs, MPI_FLOAT, ROOT, MPI_COMM_WORLD);
      MPI_Gatherv(v_r[isp], np[isp], MPI_FLOAT, vr_tot[isp], counts, 
         displs, MPI_FLOAT, ROOT, MPI_COMM_WORLD);
      MPI_Gatherv(v_theta[isp], np[isp], MPI_FLOAT, vtheta_tot[isp], counts, 
         displs, MPI_FLOAT, ROOT, MPI_COMM_WORLD);
      MPI_Gatherv(v_z[isp], np[isp], MPI_FLOAT, vz_tot[isp], counts, 
         displs, MPI_FLOAT, ROOT, MPI_COMM_WORLD);

     /* MPI_Barrier(MPI_COMM_WORLD);*/
  }

  if (my_rank == ROOT) { /* Only ROOT proc. dumps to the file */
 
    printf("np_tot[0] = %d\n", np_tot[0]);
    printf("proc 0:  np[0] = %d\n", np[0]);
    if(nsp > 1)
      {
	printf("np_tot[1] = %d\n", np_tot[1]);
	printf("proc 0:  np[1] = %d\n", np[1]);
      }

    printf("sp_n_tot[0][ng/2] = %lf\n", sp_n_tot[0][ng/2]);
    if(nsp > 1)
      {
	printf("sp_n_tot[1][ng/2] = %lf\n", sp_n_tot[1][ng/2]);
      }
    printf("proc 0: sp_n[0][ng/2] = %lf\n", sp_n[0][ng/2]/vol[ng/2]);
    if(nsp > 1)
      {
	printf("proc 0: sp_n[1][ng/2] = %lf\n", sp_n[1][ng/2]/vol[ng/2]);
      }

    if (RT_flag) XGWrite(RevisionR, 1, 4, DMPFile, "char");
    else XGWrite(Revision, 1, 4, DMPFile, "char");
    
    ftemp = atime;
    XGWrite(&ftemp, 4, 1, DMPFile, SCALAR_CHAR);
    XGWrite(&sigma, 4, 1, DMPFile, SCALAR_CHAR);
    XGWrite(&q_0, 4, 1, DMPFile, SCALAR_CHAR);
    XGWrite(&q_1, 4, 1, DMPFile, SCALAR_CHAR);
    XGWrite(&q_2, 4, 1, DMPFile, SCALAR_CHAR);
    XGWrite(&q_3, 4, 1, DMPFile, SCALAR_CHAR);
    XGWrite(&nsp, 4, 1, DMPFile, "int");
    XGWrite(iwall_tot, 4, nsp, DMPFile, SCALAR_CHAR);
    XGWrite(np_tot, 4, nsp, DMPFile, "int");
    XGWrite(&nc2p, 4, 1, DMPFile, SCALAR_CHAR);
    
    length = r1 - r0;
    for (isp=0; isp<nsp; isp++) {
      for (i=0; i<np_tot[isp]; i++) {
        ftemp = (r_tot[isp][i] -r0)/length;
        XGWrite(&ftemp, 4, 1, DMPFile, SCALAR_CHAR);
        ftemp = vscale*vr_tot[isp][i];
        XGWrite(&ftemp, 4, 1, DMPFile, SCALAR_CHAR);
        ftemp = vscale*vtheta_tot[isp][i];
        XGWrite(&ftemp, 4, 1, DMPFile, SCALAR_CHAR);
        ftemp = vscale*vz_tot[isp][i];
        XGWrite(&ftemp, 4, 1, DMPFile, SCALAR_CHAR);
      }
    }
    /*------ Save data for Radiation Transport --------*/

    if (RT_flag) {
      for (i=0;i<nc_RT;i++) {
        ftemp=sp_ex[i];
        XGWrite(&ftemp, 4, 1, DMPFile, SCALAR_CHAR);
        ftemp=sp_exm[i];
        XGWrite(&ftemp, 4, 1, DMPFile, SCALAR_CHAR);
      }
      for (i=0;i<2;i++) {
        for (j=0;j<nc_RT;j++) {
          ftemp=ex_rate_show[i][j];
          XGWrite(&ftemp, 4, 1, DMPFile, SCALAR_CHAR);
        }
      }
      ftemp=(SCALAR) it_rate;
      XGWrite(&ftemp, 4, 1, DMPFile, SCALAR_CHAR);
    }

    fclose(DMPFile);
    /***  Then write the number of particles and Ez in ascii  *****/
    strcpy(dropfile, filename);
    strcat(dropfile, ".drp");

    DropFile = fopen(dropfile, "a");
    fprintf(DropFile, "%e ", atime);
    for(i=0; i<nsp; i++) fprintf(DropFile, "%d ", np_tot[i]);
    fprintf(DropFile, "%lf ", E_z/(ptopn*dr));
    fprintf(DropFile, "%lf ", I_z);
/* 
    if (RT_flag) {
      length=ftemp=0;
      for (i=0;i<nc_RT;i++) {
        temp=2.0*i+1;
        length+=sp_ex[i]*temp;
        ftemp+=sp_exm[i]*temp;
      }
      temp=(SCALAR)(nc_RT); temp*=temp;
      length*=dntod/temp;
      ftemp *=dntod/temp;
      for(i=0; i<nsp; i++) fprintf(DropFile, "%e ",np_tot[i]*nc2p/reactor_vol);
      fprintf(DropFile, "%e %e ", length, ftemp);
    }
*/
    fprintf(DropFile, "\n");
    fclose(DropFile);
    printf("Dump done\n");
  }
#else
  if ((DMPFile = fopen(filename, "w")) == NULL)
  {
    puts("Dump: open failed");
    return;
  }
  
  printf("Start Dump\n");
  printf("np[0] = %d\n", np[0]);
  printf("np[1] = %d\n", np[1]);

  printf("sp_n[0][ng/2] = %lf\n", sp_n[0][ng/2]);
  printf("sp_n[1][ng/2] = %lf\n", sp_n[1][ng/2]);
  
  if (RT_flag) XGWrite(RevisionR, 1, 4, DMPFile, "char");
  else XGWrite(Revision, 1, 4, DMPFile, "char");
  ftemp = atime;
  XGWrite(&ftemp, 4, 1, DMPFile, SCALAR_CHAR);
  XGWrite(&sigma, 4, 1, DMPFile, SCALAR_CHAR);
  XGWrite(&q_0, 4, 1, DMPFile, SCALAR_CHAR);
  XGWrite(&q_1, 4, 1, DMPFile, SCALAR_CHAR);
  XGWrite(&q_2, 4, 1, DMPFile, SCALAR_CHAR);
  XGWrite(&q_3, 4, 1, DMPFile, SCALAR_CHAR);
  XGWrite(&nsp, 4, 1, DMPFile, "int");
  XGWrite(iwall, 4, nsp, DMPFile, SCALAR_CHAR);
  XGWrite(np, 4, nsp, DMPFile, "int");
  XGWrite(&nc2p, 4, 1, DMPFile, SCALAR_CHAR);
  
  length = r1 - r0;
  for (j=0; j<nsp; j++)
  {
    for (i=0; i<np[j]; i++)
    {
      ftemp = (r[j][i] -r0)/length;
      XGWrite(&ftemp, 4, 1, DMPFile, SCALAR_CHAR);
      ftemp = v_r[j][i]*vscale;
      XGWrite(&ftemp, 4, 1, DMPFile, SCALAR_CHAR);
      ftemp = v_theta[j][i]*vscale;
      XGWrite(&ftemp, 4, 1, DMPFile, SCALAR_CHAR);
      ftemp = v_z[j][i]*vscale;
      XGWrite(&ftemp, 4, 1, DMPFile, SCALAR_CHAR);
    }
  }
  /*------ Save data for Radiation Transport --------*/
  if (RT_flag) {
    for (i=0;i<nc_RT;i++) {
      ftemp=sp_ex[i];
      XGWrite(&ftemp, 4, 1, DMPFile, SCALAR_CHAR);
      ftemp=sp_exm[i];
      XGWrite(&ftemp, 4, 1, DMPFile, SCALAR_CHAR);
    }
    for (i=0;i<2;i++) {
      for (j=0;j<nc_RT;j++) {
        ftemp=ex_rate_show[i][j];
        XGWrite(&ftemp, 4, 1, DMPFile, SCALAR_CHAR);
      }
    }
    ftemp=(SCALAR) it_rate;
    XGWrite(&ftemp, 4, 1, DMPFile, SCALAR_CHAR);
  }
  fclose(DMPFile);
  /***  Then write the number of particles and Ez in ascii  *****/
  strcpy(dropfile, filename);
  strcat(dropfile, ".drp");

  DropFile = fopen(dropfile, "a");
  fprintf(DropFile, "%e ", atime);
  for(i=0; i<nsp; i++) fprintf(DropFile, "%d ", np[i]);
  fprintf(DropFile, "%lf ", E_z/(ptopn*dr));
  fprintf(DropFile, "%lf ", I_z);
  fprintf(DropFile, "\n");
  fclose(DropFile);

  printf("Dump done\n");
#endif
}

/****************************************************************/

void Restore(char *filename)
{
  char Rev[5];
  int i, j;
  SCALAR ftemp, length;
  FILE *DMPFile;

#ifdef MPI_1D

  SCALAR temp;
  int   isp, index, N, nnp, dest, tag, count;
  MPI_Status status;

  if (my_rank == ROOT){/* ROOT reads the dumpfile and broadcasts info. */

     if ((DMPFile = fopen(filename, "r+b")) == NULL) {
        puts("Dump: open failed");
        return;
     }

     XGRead(Rev, 1, 4, DMPFile, "char");
     for (i=0; i<4; i++)
       if (Rev[i]!=Revision[i] && Rev[i]!=RevisionR[i]) {
         puts("Incompatible dump file version");
         exit(1);
       }

     XGRead(&ftemp, 4, 1, DMPFile, SCALAR_CHAR);
     atime = ftemp;
     XGRead(&sigma, 4, 1, DMPFile, SCALAR_CHAR);
     XGRead(&q_0, 4, 1, DMPFile, SCALAR_CHAR);
     XGRead(&q_1, 4, 1, DMPFile, SCALAR_CHAR);
     XGRead(&q_2, 4, 1, DMPFile, SCALAR_CHAR);
     XGRead(&q_3, 4, 1, DMPFile, SCALAR_CHAR);
     XGRead(&nsp, 4, 1, DMPFile, "int");
     XGRead(iwall_tot, 4, nsp, DMPFile, SCALAR_CHAR);
     XGRead(np_tot, 4, nsp, DMPFile, "int");
     XGRead(&nc2p, 4, 1, DMPFile, SCALAR_CHAR);

     printf("Reading Restore:\n");
     printf("time = %lf\n", atime);
     printf("nsp = %d\n", nsp);
     printf("np_tot[0] = %d\n", np_tot[0]);
     printf("np_tot[1] = %d\n", np_tot[1]);

  } /* end if (my_rank == ROOT) */

  /* Send parameter info to other procs */
  MPI_Bcast(&atime, 1, MPI_DOUBLE, ROOT, MPI_COMM_WORLD);
  MPI_Bcast(&sigma, 1, MPI_FLOAT, ROOT, MPI_COMM_WORLD);
  MPI_Bcast(&q_0, 1, MPI_FLOAT, ROOT, MPI_COMM_WORLD);
  MPI_Bcast(&q_1, 1, MPI_FLOAT, ROOT, MPI_COMM_WORLD);
  MPI_Bcast(&q_2, 1, MPI_FLOAT, ROOT, MPI_COMM_WORLD);
  MPI_Bcast(&q_3, 1, MPI_FLOAT, ROOT, MPI_COMM_WORLD);
  MPI_Bcast(&iwall_tot, 1, MPI_FLOAT, ROOT, MPI_COMM_WORLD);
  MPI_Bcast(&np_tot, nsp, MPI_INT, ROOT, MPI_COMM_WORLD);
  MPI_Bcast(&nsp, 1, MPI_INT, ROOT, MPI_COMM_WORLD);
  MPI_Bcast(&nc2p, 1, MPI_FLOAT, ROOT, MPI_COMM_WORLD);

  oldsigma=sigma;   /* This is a cludge until I can put oldsigma into dump */
  length = r1 - r0;
     
  for (isp=0; isp<nsp; isp++) {
   if(my_rank == ROOT){
     for (i=0; i<np_tot[isp]; i++) {
        XGRead(&ftemp, 4, 1, DMPFile, SCALAR_CHAR);
        r_tot[isp][i] = ftemp*length + r0;
        XGRead(&ftemp, 4, 1, DMPFile, SCALAR_CHAR);
        vr_tot[isp][i]= ftemp/vscale;
        XGRead(&ftemp, 4, 1, DMPFile, SCALAR_CHAR);
        vtheta_tot[isp][i]= ftemp/vscale;
        XGRead(&ftemp, 4, 1, DMPFile, SCALAR_CHAR);
        vz_tot[isp][i]= ftemp/vscale;
     }

      /* Divvy up particles and send to various procs */
      nnp = np_tot[isp];
      np[isp] = floor(((SCALAR) np_tot[isp])/num_procs + 0.5);
      N = np[isp];
      printf("proc 1 to proc %d: isp = %d; np = %d\n",num_procs-1, isp, np[isp]);

      for(dest = 1; dest < num_procs; dest++){
        /* Randomly select N particles and move them to end of array */
        for(j=0; j< N; j++) {
           index= nnp*frand();
           nnp--;
           temp = r_tot[isp][nnp];
           r_tot[isp][nnp] = r_tot[isp][index];
           r_tot[isp][index] = temp;

           temp = vr_tot[isp][nnp];
           vr_tot[isp][nnp] = vr_tot[isp][index];
           vr_tot[isp][index] = temp;

           temp = vtheta_tot[isp][nnp];
           vtheta_tot[isp][nnp] = vtheta_tot[isp][index];
           vtheta_tot[isp][index] = temp;

           temp = vz_tot[isp][nnp];
           vz_tot[isp][nnp] = vz_tot[isp][index];
           vz_tot[isp][index] = temp;
        }

        for(j=0; j<N; j++) {
           r[isp][j] = r_tot[isp][nnp+j];
           v_r[isp][j] = vr_tot[isp][nnp+j];
           v_theta[isp][j] = vtheta_tot[isp][nnp+j];
           v_z[isp][j] = vz_tot[isp][nnp+j];
        }

        tag = 10 + isp;
        MPI_Send(&np[isp],1, MPI_INT,dest,tag,MPI_COMM_WORLD);
        tag = 100 + isp;
        count = np[isp];
        MPI_Send(r[isp], count, MPI_FLOAT,dest,tag,MPI_COMM_WORLD);
        tag = 200 + isp;
        MPI_Send(v_r[isp],count, MPI_FLOAT,dest,tag,MPI_COMM_WORLD);
        tag = 300 + isp;
        MPI_Send(v_theta[isp],count, MPI_FLOAT,dest,tag,MPI_COMM_WORLD);
        tag = 400 + isp;
        MPI_Send(v_z[isp],count, MPI_FLOAT,dest,tag,MPI_COMM_WORLD);

     }/* ends for(dest...) */

     /* Rest of particles assigned to ROOT proc. */
     for(j=0; j<nnp; j++) {
         r[isp][j] = r_tot[isp][j];
         v_r[isp][j] = vr_tot[isp][j];
         v_theta[isp][j] = vtheta_tot[isp][j];
         v_z[isp][j] = vz_tot[isp][j];
     }
     np[isp] = nnp; /* Note: nnp does not have to equal N */

     printf("proc 0: isp = %d; np = %d\n", isp, np[isp]);

   }/* ends if(my_rank == ROOT) */
   else{/* my_rank != ROOT */
     tag = 10 + isp;
     MPI_Recv(&np[isp],1, MPI_INT,ROOT,tag,MPI_COMM_WORLD,&status);
     tag = 100 + isp;
     count = np[isp];
     MPI_Recv(r[isp], count, MPI_FLOAT,ROOT,tag,MPI_COMM_WORLD,&status);
     tag = 200 + isp;
     MPI_Recv(v_r[isp],count, MPI_FLOAT,ROOT,tag,MPI_COMM_WORLD,&status);
     tag = 300 + isp;
     MPI_Recv(v_theta[isp],count, MPI_FLOAT,ROOT,tag,MPI_COMM_WORLD,&status);
     tag = 400 + isp;
     MPI_Recv(v_z[isp],count, MPI_FLOAT,ROOT,tag,MPI_COMM_WORLD,&status);
   }
 }/* ends for(isp...) */

 if (my_rank == ROOT) {
   /*------ Load data for Radiation Transport --------*/
     if (Rev[2] == 'R' && RT_flag) {
       for (i=0;i<nc_RT;i++) {
         XGRead(&ftemp, 4, 1, DMPFile, SCALAR_CHAR);
         sp_ex[i]=ftemp;
         XGRead(&ftemp, 4, 1, DMPFile, SCALAR_CHAR);
         sp_exm[i]=ftemp;
       }
       for (i=0;i<2;i++) {
         for (j=0;j<nc_RT;j++) {
           XGRead(&ftemp, 4, 1, DMPFile, SCALAR_CHAR);
           ex_rate_show[i][j]=ftemp;
         }
       }
       XGRead(&ftemp, 4, 1, DMPFile, SCALAR_CHAR);
       it_rate=(int) ftemp;
     }
   /*-------------------------------------------------*/
   fclose(DMPFile);
   printf("End Restore\n");
 }

#else
  if ((DMPFile = fopen(filename, "r+b")) == NULL)
  {
    puts("Restore: open failed");
    return;
  }
  
  XGRead(Rev, 1, 4, DMPFile, "char");
  for (i=0; i<4; i++)
    if (Rev[i]!=Revision[i] && Rev[i]!=RevisionR[i]) {
      puts("Incompatible dump file version");
      putchar(7);
      exit(1);
    }

  XGRead(&ftemp, 4, 1, DMPFile, SCALAR_CHAR);
  atime = ftemp;
  XGRead(&sigma, 4, 1, DMPFile, SCALAR_CHAR);
  XGRead(&q_0, 4, 1, DMPFile, SCALAR_CHAR);
  XGRead(&q_1, 4, 1, DMPFile, SCALAR_CHAR);
  XGRead(&q_2, 4, 1, DMPFile, SCALAR_CHAR);
  XGRead(&q_3, 4, 1, DMPFile, SCALAR_CHAR);
  XGRead(&nsp, 4, 1, DMPFile, "int");
  XGRead(iwall, 4, nsp, DMPFile, SCALAR_CHAR);
  XGRead(np, 4, nsp, DMPFile, "int");
  XGRead(&nc2p, 4, 1, DMPFile, SCALAR_CHAR);
  
  printf("Reading Restore:\n");
  printf("time = %lf\n", atime);
  printf("nsp = %d\n", nsp);
  printf("np[0] = %d\n", np[0]);
  printf("np[1] = %d\n", np[1]);
     
  oldsigma=sigma;   /* This is a cludge until I can put oldsigma into dump */
  length = r1 - r0;
  for (j=0; j<nsp; j++)
  {
    for (i=0; i<np[j]; i++)
    {
      XGRead(&ftemp, 4, 1, DMPFile, SCALAR_CHAR);
      r[j][i] = length*ftemp +r0;
      XGRead(&ftemp, 4, 1, DMPFile, SCALAR_CHAR);
      v_r[j][i] = ftemp/vscale;
      XGRead(&ftemp, 4, 1, DMPFile, SCALAR_CHAR);
      v_theta[j][i] = ftemp/vscale;
      XGRead(&ftemp, 4, 1, DMPFile, SCALAR_CHAR);
      v_z[j][i] = ftemp/vscale;
    }
  }

  /*------ Load data for Radiation Transport --------*/
  if (Rev[2] == 'R' && RT_flag) {
    for (i=0;i<nc_RT;i++) {
      XGRead(&ftemp, 4, 1, DMPFile, SCALAR_CHAR);
      sp_ex[i]=ftemp;
      XGRead(&ftemp, 4, 1, DMPFile, SCALAR_CHAR);
      sp_exm[i]=ftemp;
    }
    for (i=0;i<2;i++) {
      for (j=0;j<nc_RT;j++) {
        XGRead(&ftemp, 4, 1, DMPFile, SCALAR_CHAR);
        ex_rate_show[i][j]=ftemp;
      }
    }
    XGRead(&ftemp, 4, 1, DMPFile, SCALAR_CHAR);
    it_rate=(int) ftemp;
  }

  fclose(DMPFile);
  printf("End Restore\n");
#endif

}

void Quit(void)
{
#ifdef MPI_1D
  endwtime = MPI_Wtime();
  if (my_rank == ROOT){
    printf("wall clock run time = %lf\n", endwtime-time0);
    printf("init time = %lf\n", time0-startwtime);
  }
  
  if (my_rank == ROOT) printf("Finalizing MPI\n");
  MPI_Finalize();
  exit(0);
#else
  endwtime = clock();
  printf("wall clock run time = %lf\n", (SCALAR)(endwtime-time0)/CLOCKS_PER_SEC);
  printf("init time = %lf\n", (SCALAR)(time0-startwtime)/CLOCKS_PER_SEC);
#endif
}
