#include "pdc1.h"
#include "xgrafix.h"

SCALAR F(SCALAR);

/*****************************************************************
** Calculates a maxwellian velocity distribution for gas atoms 
** Used in loading new ions created by ionisation or in charge
** exchange collisions
****************************************************************/
void maxwellv(SCALAR *vx_local, SCALAR *vy_local, SCALAR *vz_local, SCALAR vth)
{
  static int nvel, init_flag= 1;
  static SCALAR *vsave_x, *vsave_y, *vsave_z;
  SCALAR nvts = 3.3;
          
  int i, n;
  SCALAR vmag, aphi, dv, rr, sintheta, costheta;
	          
  if (init_flag) 
  {
     nvel= 1./(1-F(nvts));
     if(nvel > 100000)
   #ifdef MPI_1D
     if (my_rank == ROOT)
       puts("Warning: Your choice of nvts has made nvel > 1e5"); 
   #else
     puts("Warning: Your choice of nvts has made nvel > 1e5"); 
   #endif
     dv = sqrt(PI)/(4.0*nvel);

     vsave_x= (SCALAR *) malloc(nvel*sizeof(SCALAR));
     vsave_y= (SCALAR *) malloc(nvel*sizeof(SCALAR));
     vsave_z= (SCALAR *) malloc(nvel*sizeof(SCALAR));
     init_flag= 0;
     i=n=0;
     for (n=0; n<nvel; n++) 
     {
        rr=(1.0*n)/nvel;
	while (F(i*dv)< rr) i++;
	vmag=i*dv;
	aphi=2*PI*frand();
	costheta = 1-2*frand();
	sintheta = sqrt(1-costheta*costheta);
	vsave_x[n] = vmag*sintheta*cos(aphi);
	vsave_y[n] = vmag*sintheta*sin(aphi);
	vsave_z[n] = vmag*costheta;
     }
  }
  n = (nvel-1)*frand();
  *vx_local = vth*vsave_x[n];
  *vy_local = vth*vsave_y[n];
  *vz_local = vth*vsave_z[n];
}
/**************************************************************/
SCALAR F(SCALAR v)
{
  return(-2*v*exp(-v*v)/sqrt(PI) +erf(v));
}
/*---------------------------------------------------------------------------*/
/* This routine swaps 2 array elements */
void swap (SCALAR v[], int i, int j)
{
      SCALAR temp;
 
      temp=v[i];  
      v[i]=v[j];  
      v[j]=temp;  
}
/*---------------------------------------------------------------------------*/
